"""This module lists all required environment variables used by NOVA-Server

Author:
    Dominik Schiller <dominik.schiller@uni-a.de>
Date:
    13.09.2023

"""

NOVA_SERVER_HOST = "NOVA_SERVER_HOST"
NOVA_SERVER_PORT = "NOVA_SERVER_PORT"
NOVA_SERVER_CML_DIR = "NOVA_SERVER_CML_DIR"
NOVA_SERVER_DATA_DIR = "NOVA_SERVER_DATA_DIR"
NOVA_SERVER_CACHE_DIR = "NOVA_SERVER_CACHE_DIR"
NOVA_SERVER_TMP_DIR = "NOVA_SERVER_TMP_DIR"
NOVA_SERVER_LOG_DIR = "NOVA_SERVER_LOG_DIR"
